﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

//////////////////////////////////////////////////////////////////
//【各ブロック単位の移動操作や完了判定処理に使用するクラス】
// スタート時のブロック配置、ブロックの移動操作処理、ブロックの完了判定処理
//////////////////////////////////////////////////////////////////
public class BlockController : MonoBehaviour
{
    //【ブロックの位置番号】
    //　１　２　３
    //　４　５　６
    //　７　８　９

    /////////////////////////////////
    // 【変数の設定】
    /////////////////////////////////

    //【定数の設定】（レベル設計で変更可能）
    const float LERP_FACTOR = 0.3f;  //ブロックに滑らかな移動をさせる係数（0-1）

    //【Unity Editorから設定するPublic変数】
    public BlockManager class_BlockManager;  // BlockManagerの関数呼び出し用
    public int block_Num;  // ブロック番号
                           // 本スクリプトが関連付けられたブロックのブロック番号（1-8）

    //【このプログラム内で使用するPrivate変数】
    private bool isClickBlock = false;
    // 本スクリプトが関連付けられたブロック上でマウス左ボタンが押され続けている場合にtrue
    private bool isMoveBlock = false;
    // 本スクリプトが関連付けられたブロックが移動中の場合にtrue

    private int posNum_Now;　// 本スクリプトが関連付けられたブロックの現在の位置番号（1-9）
    private int posNum_Target;  // ブロックの移動予定の位置番号（1-9）
    private int dif_PosNumber;   // ブロックの移動予定の位置番号と現在の位置番号の差分
                                 //（3,-3:上下移動、1,-1:左右移動）

    private Vector3 blockPos_Now;  // 本スクリプトが関連付けられたブロックの移動前の位置座標
    private Vector3 blockPos_Target;  // ブロックの移動予定の位置座標
    private Vector3 mousePos_Start;  // マウス左ボタンが押された時のマウスポインタの画面上の位置


    /////////////////////////////////
    // 【関数の設定】
    /////////////////////////////////

    //【ゲーム開始時に１回だけ動く関数】
    //（Unityの標準関数）
    void Start()
    {
        // 本スクリプトが関連付けられたブロックのスタート時の位置番号をBlockManagerから取得
        posNum_Now = class_BlockManager.OnGet_PosNum_Initial(block_Num);

        // ブロックのスタート時の位置番号から現在の位置座標を設定
        switch (posNum_Now)
        {
            case 1:
                blockPos_Now = new Vector3(-1.5f, 1.5f, 0f);
                break;
            case 2:
                blockPos_Now = new Vector3(0f, 1.5f, 0f);
                break;
            case 3:
                blockPos_Now = new Vector3(1.5f, 1.5f, 0f);
                break;
            case 4:
                blockPos_Now = new Vector3(-1.5f, 0f, 0f);
                break;
            case 5:
                blockPos_Now = new Vector3(0f, 0f, 0f);
                break;
            case 6:
                blockPos_Now = new Vector3(1.5f, 0f, 0f);
                break;
            case 7:
                blockPos_Now = new Vector3(-1.5f, -1.5f, 0f);
                break;
            case 8:
                blockPos_Now = new Vector3(0f, -1.5f, 0f);
                break;
        }

        // 本スクリプトが関連付けられたブロックをスタート時の位置に移動
        transform.position = blockPos_Now;
    }


    //【毎フレームに１度動く関数】
    //（Unityの標準関数）
    void Update()
    {
        // isClickBlockがtrueの場合（マウス左ボタンが押され続けている）
        //（本スクリプトが関連付けられたブロック上でマウス左ボタンが押され続けている場合）
        if (isClickBlock)
        {
            // 現在のマウスポインタの画面上の位置
            Vector3 mousePos_Now = Camera.main.ScreenToWorldPoint(Input.mousePosition);
            mousePos_Now.z = 0;

            // マウス左ボタンが押された位置と現在の位置の差分
            //（マウス左ボタンが押された位置からのマウスの移動方向と距離）
            Vector3 mousePos_Dif = mousePos_Now - mousePos_Start;

            // 本スクリプトが関連付けられているブロックの現在の位置にマウスの移動方向と距離を足す
            //（マウスの移動に合わせてブロックを移動させるため）
            Vector3 blockPos_Move = blockPos_Now + mousePos_Dif;

            // ブロックの移動範囲の制限設定（X座標方向に１マスだけ動く様に）
            // ブロックが左に移動可能な場合
            // 左右に移動不可能な場合はTargetとNowが同じであり移動制限となる
            if (blockPos_Now.x >= blockPos_Target.x)
            {
                // ブロックのX座標方向の移動は左の移動予定位置から右の現在位置までの範囲内とする
                blockPos_Move.x =
                    Mathf.Clamp(blockPos_Move.x, blockPos_Target.x, blockPos_Now.x);
            }
            // ブロックが右に移動可能な場合
            else
            {
                // ブロックのX座標方向の移動は左の現在位置から右の移動予定位置までの範囲内とする
                blockPos_Move.x =
                    Mathf.Clamp(blockPos_Move.x, blockPos_Now.x, blockPos_Target.x);
            }

            // ブロックの移動範囲の制限設定（Y座標方向に１マスだけ動く様に）
            // ブロックが下に移動可能な場合
            // 上下に移動不可能な場合はTargetとNowが同じであり移動制限となる
            if (blockPos_Now.y >= blockPos_Target.y)
            {
                // ブロックのY座標方向の移動は下の移動予定位置から上の現在位置までの範囲内とする
                blockPos_Move.y =
                    Mathf.Clamp(blockPos_Move.y, blockPos_Target.y, blockPos_Now.y);
            }
            // ブロックが上に移動可能な場合
            else
            {
                // ブロックのY座標方向の移動は下の現在位置から上の移動予定位置までの範囲内とする
                blockPos_Move.y =
                    Mathf.Clamp(blockPos_Move.y, blockPos_Now.y, blockPos_Target.y);
            }

            // 本スクリプトが関連付けられたブロックを、移動制限設定をされたマウスの位置に滑らかに
            // 移動させる（瞬間移動ではない）
            transform.position =
                Vector3.Lerp(transform.position, blockPos_Move, LERP_FACTOR);
        }


        // isClickBlockがfalseの場合
        //（本スクリプトが関連付けられたブロック上でマウス左ボタンが押されていない場合）
        //（マウス左ボタンが押されていない、マウス左ボタンが放された）
        else
        {
            // isMoveBlockがtrueの場合（本スクリプトが関連付けられたブロックが移動中の場合）
            //（マウス左ボタンが放されているので、移動予定位置に移動中か、元々の位置に戻っている）
            if (isMoveBlock)
            {
                // 上か下に移動中
                if (dif_PosNumber == 3 || dif_PosNumber == -3)
                {
                    // 移動予定位置と移動中のブロックの現在位置の距離が0.75以下の場合
                    //（元々の位置から移動予定位置の半分の位置を超えた場合：移動予定位置に移動）
                    if (Mathf.Abs(blockPos_Target.y - transform.position.y) <= 0.75f)
                    {
                        // 本スクリプトが関連付けられたブロックを、移動予定位置に滑らかに
                        // 移動させる（瞬間移動ではない） 
                        transform.position =
                            Vector3.Lerp(transform.position, blockPos_Target, LERP_FACTOR);

                        // ブロックの位置から移動予定位置までの距離が0.1未満になったら
                        //（Absは絶対値を取得する関数）
                        if (Mathf.Abs(blockPos_Target.y - transform.position.y) < 0.1f)
                        {
                            // ブロックの移動予定位置への移動処理と移動終了処理
                            OnMove_BlockTarget();
                        }
                    }

                    // 移動予定位置と移動中のブロックの現在位置の距離が0.75より大きい場合
                    // （元々の位置から移動予定位置の半分の位置を超えてない場合：元々の位置に戻る）
                    else if (Mathf.Abs(blockPos_Target.y - transform.position.y) > 0.75f)
                    {
                        // 本スクリプトが関連付けられたブロックを、元々の位置に滑らかに移動させる
                        //（瞬間移動ではない）
                        transform.position =
                            Vector3.Lerp(transform.position, blockPos_Now, LERP_FACTOR);

                        // ブロックの位置から移動予定位置までの距離が1.4より大きくなったら
                        //（Absは絶対値を取得する関数）
                        if (Mathf.Abs(blockPos_Target.y - transform.position.y) > 1.4f)
                        {
                            // ブロックの位置を元々の位置に戻す
                            transform.position = blockPos_Now;

                            // 本スクリプトが関連付けられたブロックの移動が終了したのでfalse
                            isMoveBlock = false;
                        }
                    }
                }

                // 左か右に移動中
                else if (dif_PosNumber == 1 || dif_PosNumber == -1)
                {
                    // 移動予定位置と移動中のブロックの現在位置の距離が0.75以下の場合
                    // （元々の位置から移動予定位置の半分の位置を超えた場合：移動予定位置に移動）
                    if (Mathf.Abs(blockPos_Target.x - transform.position.x) <= 0.75f)
                    {
                        // 本スクリプトが関連付けられたブロックを、移動予定位置に滑らかに移動させる
                        //（瞬間移動ではない）
                        transform.position =
                            Vector3.Lerp(transform.position, blockPos_Target, LERP_FACTOR);

                        // ブロックの位置から移動予定位置までの距離が0.1未満になったら
                        //（Absは絶対値を取得する関数）
                        if (Mathf.Abs(blockPos_Target.x - transform.position.x) < 0.1f)
                        {
                            // ブロックの移動予定位置への移動処理と移動終了処理
                            OnMove_BlockTarget();
                        }
                    }

                    // 移動予定位置と移動中のブロックの現在位置の距離が0.75より大きい場合
                    // （元々の位置から移動予定位置の半分の位置を超えてない場合：元々の位置に戻る）
                    else if (Mathf.Abs(blockPos_Target.x - transform.position.x) > 0.75f)
                    {
                        // 本スクリプトが関連付けられたブロックを、元々の位置に滑らかに移動させる
                        //（瞬間移動ではない）
                        transform.position =
                            Vector3.Lerp(transform.position, blockPos_Now, LERP_FACTOR);

                        // ブロックの位置から移動予定位置までの距離が1.4より大きくなったら
                        //（Absは絶対値を取得する関数）
                        if (Mathf.Abs(blockPos_Target.x - transform.position.x) > 1.4f)
                        {
                            // ブロックの位置を元々の位置に戻す
                            transform.position = blockPos_Now;

                            // 本スクリプトが関連付けられたブロックの移動が終了したのでfalse
                            isMoveBlock = false;
                        }
                    }
                }
            }
        }
    }


    //【本スクリプトが関連付けられたブロック上でマウス左ボタンが押された場合に呼ばれる関数】
    public void OnMouseDown_MoveBlock()
    {
        // 本スクリプトが関連付けられたブロック上でマウス左ボタンが押されたのでtrue
        isClickBlock = true;
        // 本スクリプトが関連付けられたブロックが移動中なのでtrue
        isMoveBlock = true;

        // 現在ブロックが移動可能な位置番号を、クリックしたブロックの移動予定の位置番号とする
        posNum_Target = class_BlockManager.OnGet_PosNum_MovePossible();
        // ブロックの移動予定の位置番号と本スクリプトが関連付けられたブロックの現在の位置番号の差分
        dif_PosNumber = posNum_Target - posNum_Now;
        // クリックしたブロックの移動予定位置を現在の位置を基準に算出する
        blockPos_Target = blockPos_Now;

        // クリックしたブロックが下に移動可能な場合
        if (dif_PosNumber == 3)
        {
            // クリックしたブロックの移動予定位置を現在の位置から１つ下の位置にする
            blockPos_Target.y -= 1.5f;
        }
        // クリックしたブロックが上に移動可能な場合
        else if (dif_PosNumber == -3)
        {
            // クリックしたブロックの移動予定位置を現在の位置から１つ上の位置にする
            blockPos_Target.y += 1.5f;
        }
        // クリックしたブロックが右に移動可能な場合
        //（現在の位置番号が３か６の場合は右には移動できない）
        else if (dif_PosNumber == 1 && (posNum_Now != 3 && posNum_Now != 6))
        {
            // クリックしたブロックの移動予定位置を現在の位置から１つ右の位置にする
            blockPos_Target.x += 1.5f;
        }
        // クリックしたブロックが左に移動可能な場合
        //（現在の位置番号が４か７の場合は左には移動できない）
        else if (dif_PosNumber == -1 && (posNum_Now != 4 && posNum_Now != 7))
        {
            // クリックしたブロックの移動予定位置を現在の位置から１つ左の位置にする
            blockPos_Target.x -= 1.5f;
        }
        // クリックしたブロックが移動不可能な場合
        else
        {
            // クリックしているブロックは移動不可能なので移動用の変数をクリアしてしまう
            isClickBlock = false;
            isMoveBlock = false;
            return;
        }

        // マウス左ボタンが押された画面上の位置
        mousePos_Start = Camera.main.ScreenToWorldPoint(Input.mousePosition);
        mousePos_Start.z = 0;
    }


    //【マウス左ボタンが押されたあと、マウス左ボタンが放された場合に呼ばれる関数】
    public void OnMouseUp_MoveBlock()
    {
        // マウス左ボタンが放されたのでfalse
        isClickBlock = false;
    }


    //【本スクリプトが関連付けられたブロックの完了判定処理関数】
    // 本スクリプトが関連付けられたブロックの現在の位置番号がブロック番号と同じかを返信
    //（ブロック番号はブロックの最終的な位置の位置番号を使用しているので、
    // 現在の位置番号と比較することで、そのブロックが最終的な位置に現在あるのかが分かる）
    public bool OnGet_FinishStatus()
    {
        return (block_Num == posNum_Now);
    }


    //【ブロックの移動予定位置への移動処理と移動終了処理】
    private void OnMove_BlockTarget()
    {
        // ブロックの位置を移動予定位置にする
        transform.position = blockPos_Target;

        // 次に移動可能な位置番号がブロックが元々いた位置番号になることを
        // BlockManagerに知らせる
        class_BlockManager.OnSet_PosNum_MovePossible(posNum_Now);

        // 移動した本ブロックの位置番号と位置座標が移動先の位置番号と位置座標となる
        posNum_Now = posNum_Target;
        blockPos_Now = blockPos_Target;

        // 本スクリプトが関連付けられたブロックの移動が終了したのでfalse
        isMoveBlock = false;

        // ブロックの移動終了後BlockManagerで終了判定処理をする
        class_BlockManager.Check_Complete();
    }
}
